==> ToolChain_STM32_1.0.1.tar.bz: ToolChain_STM32/Libraries/stm32_io.c <==
/*{ stm32_io.c ***********************************************
 
 * Written by Gregor Rebel 2010-2011
 *
 * Basic set of helper functions.
 * 
}*/

#include "stm32_io.h"

     u16_t* arraySet(u16_t* Array, u16_t Value, u32_t Amount) {                // fill 16-bit array with given value                                      
    u16_t* Writer16 = Array;
    
    while (Amount-- > 0) {
      *Writer16++ = Value;
    }
    
    return Array;
}
       void Assert(volatile bool Condition, volatile ErrorCode_e ErrorCode) {  // block endless in case Condition is false
    while (!Condition) ErrorCode = ErrorCode;
    //if (!Condition)  HardFault_Handler();
}
       void initPort(GPIO_TypeDef* GPIOx, u32_t Pin, GPIOMode_TypeDef Type) {  // initializes single port bit for use as input/ output
/* Arguments:
 * GPIOx  one of GPIOA, GPIOB, GPIOC, GPIOD, GPIOE
 * Pin    0..15
 * Type   one of GPIO_Mode_AIN, 
 *               GPIO_Mode_IN_FLOATING,
 *               GPIO_Mode_IPD,
 *               GPIO_Mode_IPU,
 *               GPIO_Mode_Out_OD,
 *               GPIO_Mode_Out_PP,
 *               GPIO_Mode_Out_OD,
 *               GPIO_Mode_AF_PP,
 *               GPIO_Mode_AF_OD
 */

    switch ( (u32_t) GPIOx) { // enable peripheral clock to this port
      case (u32_t) GPIOA: RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE); break;
      case (u32_t) GPIOB: RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB, ENABLE); break;
      case (u32_t) GPIOC: RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC, ENABLE); break;
      case (u32_t) GPIOD: RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOD, ENABLE); break;
      case (u32_t) GPIOE: RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOE, ENABLE); break;
      default: Assert(0, ec_InitPort_UnknownGPIO); break; // ERROR: unknown GPIO given!
    }
    
    GPIO_InitTypeDef GPIO_InitStructure;
    GPIO_InitStructure.GPIO_Mode  = Type;
    GPIO_InitStructure.GPIO_Pin   = 1 << Pin;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOx, &GPIO_InitStructure);
    
    switch (Type) {
      case GPIO_Mode_IPU: {
        break;
      }
      case GPIO_Mode_Out_PP: {
        GPIO_WriteBit(GPIOx, 1 << Pin, Bit_SET);
        break;
      }
      case GPIO_Mode_Out_OD: {
        portSet(GPIOx, Pin);
        break;
      }
      default: break;
    }
}
       bool checkSwitch_LowActive(GPIO_TypeDef* GPIOx, u32_t Pin) {            // checks if a low-active switch input is pressed for some time
         
  bool Pressed = FALSE;
  
  // switch-value has to be constant over a certain amount of time
  for (int Retries = 100; Retries > 0; Retries--) {
    Pressed = !portGet(GPIOx, Pin);  // assuming switch is low-active
    if (!Pressed)  break;
  }
  
  return Pressed;
}
      void* myMalloc(size_t Size) {                                            // allocates zeroed memory
    
    void* Buffer = pvPortMalloc(Size);
    Assert(Buffer != 0, ec_Malloc);
    memset(Buffer, 0, Size);
    
    return Buffer;
}
      s16_t sLimit(s16_t Value, s16_t Min, s16_t Max) {                        // sLimits a value to given minimum and maximum 
  if (Value < Min) Value = Min;
  if (Value > Max) Value = Max;
  
  return Value;
}
      u16_t toggleValue(u16_t Value, u16_t Max, u16_t Min) {                   // returns Max if Value tends to be Min, Min otherwise
    if (Value == Max)  return Min;
    if (Value == Min)  return Max;
    
    // Value is neither Min nor Max: see to which it tends to
    u16_t Dist2Min = abs(Value - Min);
    u16_t Dist2Max = abs(Max - Value);
    
    if (Dist2Min < Dist2Max)  return Max;
    return Min;
}
      u16_t uLimit(u16_t Value, u16_t Min, u16_t Max) {                        // sLimits a value to given minimum and maximum 
  if (Value < Min) Value = Min;
  if (Value > Max) Value = Max;
  
  return Value;
}
       bool circa(u32_t A, u32_t B, u8_t Distance) {                           // checks if two values are nearly the same
/* checks if B is within [A-Distance..A+Distance]
 *
 * A         first value
 * B         second values
 * Distance  distance around A
 *
 * return: TRUE = B is within given distance around A.
 */
    u32_t C = 0xffffffff;
    if (C - Distance > A)  C = A + Distance;
    if (C < B)  return FALSE;  // B < A - Distance

    C = 0;
    if (Distance < A)  C = A - Distance;
    if (C > B)  return FALSE;  // B > A + Distance
      
    return TRUE;               // A - Distance <= B <= A + Distance
}
       bool portGet(GPIO_TypeDef* GPIOx, u32_t Pin) {                          // read current value of input pin
  u32_t Data = GPIOx->IDR;
    if ( Data & (1 << Pin) ) return TRUE;
    return FALSE;
}
       void portSet(GPIO_TypeDef* GPIOx, u32_t Pin) {                          // set output pin to logical one
    GPIOx->BSRR |= 1 << Pin;
}
       void portClr(GPIO_TypeDef* GPIOx, u32_t Pin) {                          // set output pin to logical zero
    GPIOx->BRR  |= 1 << Pin;
}
       void programCalibrateUSleep() {                                         // produces software PWM usable to calibrate uSleep() 
  // Synopsis: Connect oscilloscope to pin PA1 (pin 15 on STM32F103) an check for correct period (1,1ms) and duty cycle.
  //           Adjust uSleep() accordingly, recompile, reflash and retest.
  
  initPort(GPIOA, 1, GPIO_Mode_Out_PP);
  
  while (1) {
    portSet(GPIOA, 1);
    uSleep(1000);           // 1ms high time
    portClr(GPIOA, 1);
    uSleep(100);            // 100us low time
  }
}
       void mSleep(u32_t Delay) {                                              // set task to sleep for some amount of time 
/* Arguments: 
 * Delay    delay time (msecs)
*/

    for (;Delay > 0; Delay--)
      uSleep(1000);
}

void uSleep(u32_t Delay) {                                              // set task to sleep for some amount of time 
/* Arguments: 
 * Delay    delay time (usecs)
*/

    Delay /= (1000000 / configTICK_RATE_HZ); // smallest delay depends on tick rate <-- test setting via programCalibrateUSleep()
    while (Delay > 0xffff) { // split long delays into smaller ones
      vTaskDelay(Delay & 0xffff);
      Delay -= 0xffff;
    }
    vTaskDelay(Delay);
}

==> ToolChain_STM32_1.0.11.tar.bz: ToolChain_STM32/Libraries/stm32_io.c <==
/*{ stm32_io.c ***********************************************
 
 * Written by Gregor Rebel 2010-2011
 *
 * Basic set of helper functions.
 * 
}*/

#include "stm32_io.h"

     u16_t* arraySet(u16_t* Array, u16_t Value, u32_t Amount) {                // fill 16-bit array with given value                                      
    u16_t* Writer16 = Array;
    
    while (Amount-- > 0) {
      *Writer16++ = Value;
    }
    
    return Array;
}
       void Assert(volatile bool Condition, volatile ErrorCode_e ErrorCode) {  // block endless in case Condition is false
    while (!Condition) ErrorCode = ErrorCode;
    //if (!Condition)  HardFault_Handler();
}
       void initPort(GPIO_TypeDef* GPIOx, u32_t Pin, GPIOMode_TypeDef Type) {  // initializes single port bit for use as input/ output
        initPort2(GPIOx, Pin, Type, GPIO_Speed_50MHz); 
       }
       void initPort2(GPIO_TypeDef* GPIOx, u32_t Pin, GPIOMode_TypeDef Type, GPIOSpeed_TypeDef Speed) {  // initializes single port bit for use as input/ output
/* Arguments:
 * GPIOx  one of GPIOA, GPIOB, GPIOC, GPIOD, GPIOE
 *
 * Pin    0..15
 *
 * Type   one of GPIO_Mode_AIN, 
 *               GPIO_Mode_IN_FLOATING,
 *               GPIO_Mode_IPD,
 *               GPIO_Mode_IPU,
 *               GPIO_Mode_Out_OD,
 *               GPIO_Mode_Out_PP,
 *               GPIO_Mode_Out_OD,
 *               GPIO_Mode_AF_PP,
 *               GPIO_Mode_AF_OD
 *
 * Speed  one of GPIO_Speed_10MHz,
 *               GPIO_Speed_2MHz, 
 *               GPIO_Speed_50MHz
 */

    switch ( (u32_t) GPIOx) { // enable peripheral clock to this port
      case (u32_t) GPIOA: RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE); break;
      case (u32_t) GPIOB: RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB, ENABLE); break;
      case (u32_t) GPIOC: RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC, ENABLE); break;
      case (u32_t) GPIOD: RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOD, ENABLE); break;
      case (u32_t) GPIOE: RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOE, ENABLE); break;
      default: Assert(0, ec_InitPort_UnknownGPIO); break; // ERROR: unknown GPIO given!
    }
    
    GPIO_InitTypeDef GPIO_InitStructure;
    GPIO_InitStructure.GPIO_Mode  = Type;
    GPIO_InitStructure.GPIO_Pin   = 1 << Pin;
    GPIO_InitStructure.GPIO_Speed = Speed;
    GPIO_Init(GPIOx, &GPIO_InitStructure);
    
    switch (Type) {
      case GPIO_Mode_IPU: {
        break;
      }
      case GPIO_Mode_Out_PP: {
        GPIO_WriteBit(GPIOx, 1 << Pin, Bit_SET);
        break;
      }
      case GPIO_Mode_Out_OD: {
        portSet(GPIOx, Pin);
        break;
      }
      default: break;
    }
}
       bool checkSwitch_LowActive(GPIO_TypeDef* GPIOx, u32_t Pin) {            // checks if a low-active switch input is pressed for some time
         
  bool Pressed = FALSE;
  
  // switch-value has to be constant over a certain amount of time
  for (int Retries = 100; Retries > 0; Retries--) {
    Pressed = !portGet(GPIOx, Pin);  // assuming switch is low-active
    if (!Pressed)  break;
  }
  
  return Pressed;
}
      void* myMalloc(size_t Size) {                                            // allocates zeroed memory
    
    void* Buffer = pvPortMalloc(Size);
    Assert(Buffer != 0, ec_Malloc);
    memset(Buffer, 0, Size);
    
    return Buffer;
}
      s16_t sLimit(s16_t Value, s16_t Min, s16_t Max) {                        // sLimits a value to given minimum and maximum 
  if (Value < Min) Value = Min;
  if (Value > Max) Value = Max;
  
  return Value;
}
      u16_t toggleValue(u16_t Value, u16_t Max, u16_t Min) {                   // returns Max if Value tends to be Min, Min otherwise
    if (Value == Max)  return Min;
    if (Value == Min)  return Max;
    
    // Value is neither Min nor Max: see to which it tends to
    u16_t Dist2Min = abs(Value - Min);
    u16_t Dist2Max = abs(Max - Value);
    
    if (Dist2Min < Dist2Max)  return Max;
    return Min;
}
      u16_t uLimit(u16_t Value, u16_t Min, u16_t Max) {                        // sLimits a value to given minimum and maximum 
  if (Value < Min) Value = Min;
  if (Value > Max) Value = Max;
  
  return Value;
}
       bool circa(u32_t A, u32_t B, u8_t Distance) {                           // checks if two values are nearly the same
/* checks if B is within [A-Distance..A+Distance]
 *
 * A         first value
 * B         second values
 * Distance  distance around A
 *
 * return: TRUE = B is within given distance around A.
 */
    u32_t C = 0xffffffff;
    if (C - Distance > A)  C = A + Distance;
    if (C < B)  return FALSE;  // B < A - Distance

    C = 0;
    if (Distance < A)  C = A - Distance;
    if (C > B)  return FALSE;  // B > A + Distance
      
    return TRUE;               // A - Distance <= B <= A + Distance
}
       bool portGet(GPIO_TypeDef* GPIOx, u32_t Pin) {                          // read current value of input pin
  u32_t Data = GPIOx->IDR;
    if ( Data & (1 << Pin) ) return TRUE;
    return FALSE;
}
       void portSet(GPIO_TypeDef* GPIOx, u32_t Pin) {                          // set output pin to logical one
    GPIOx->BSRR |= 1 << Pin;
}
       void portClr(GPIO_TypeDef* GPIOx, u32_t Pin) {                          // set output pin to logical zero
    GPIOx->BRR  |= 1 << Pin;
}
       void programCalibrateUSleep() {                                         // produces software PWM usable to calibrate uSleep() 
  // Synopsis: Connect oscilloscope to pin PA1 (pin 15 on STM32F103) an check for correct period (1,1ms) and duty cycle.
  //           Adjust uSleep() accordingly, recompile, reflash and retest.
  
  initPort(GPIOA, 1, GPIO_Mode_Out_PP);
  
  while (1) {
    portSet(GPIOA, 1);
    uSleep(1000);           // 1ms high time
    portClr(GPIOA, 1);
    uSleep(100);            // 100us low time
  }
}
       void mSleep(u32_t Delay) {                                              // set task to sleep for some amount of time 
/* Arguments: 
 * Delay    delay time (msecs)
*/

    for (;Delay > 0; Delay--)
      uSleep(1000);
}

void uSleep(u32_t Delay) {                                              // set task to sleep for some amount of time 
/* Arguments: 
 * Delay    delay time (usecs)
*/

    Delay /= (1000000 / configTICK_RATE_HZ); // smallest delay depends on tick rate <-- test setting via programCalibrateUSleep()
    while (Delay > 0xffff) { // split long delays into smaller ones
      vTaskDelay(Delay & 0xffff);
      Delay -= 0xffff;
    }
    vTaskDelay(Delay);
}
